@@ -14,8 +14,8 @@ class TestItemCreation(unittest.TestCase):
             wbi_core.Time('+%Y-%m-%dT%H:%M:%SZ', "P6"),
             wbi_core.Url("http://www.google.com", "P7"),
             wbi_core.MonolingualText("xxx", prop_nr="P8"),
-            wbi_core.Quantity(5, prop_nr="P9"),
-            wbi_core.Quantity(5, upper_bound=9, lower_bound=2, prop_nr="P10"),
+            wbi_core.Quantity(-5.04, prop_nr="P9"),
+            wbi_core.Quantity(5.06, upper_bound=9.99, lower_bound=-2.22, unit="Q11573", prop_nr="P10"),
             wbi_core.CommonsMedia("xxx", prop_nr="P11"),
             wbi_core.GlobeCoordinate(1.2345, 1.2345, 12, prop_nr="P12"),
             wbi_core.GeoShape("Data:xxx.map", prop_nr="P13"),
