@@ -1,7 +1,7 @@
-import unittest
 import pprint
-import sys
-from wikibaseintegrator import wdi_core, wdi_fastrun, wdi_login
+import unittest
+
+from wikibaseintegrator import wbi_core
 
 __author__ = 'Sebastian Burgstaller-Muehlbacher'
 __license__ = 'AGPLv3'
@@ -13,7 +13,7 @@ class TestItemGenerator(unittest.TestCase):
         items = ['Q408883', 'P715', 'Q18046452']
 
         login_obj = None
-        item_instances = wdi_core.WDItemEngine.generate_item_instances(items=items, login=login_obj)
+        item_instances = wbi_core.ItemEngine.generate_item_instances(items=items, login=login_obj)
 
         for qid, item in item_instances:
             print(qid)
