@@ -1,3 +1,4 @@
+import collections
 import copy
 from collections import defaultdict
 from functools import lru_cache
@@ -5,21 +6,10 @@ from itertools import chain
 
 from wikibaseintegrator.wbi_config import config
 
-example_Q14911732 = {'P1057':
-                         {'Q14911732-23F268EB-2848-4A82-A248-CF4DF6B256BC':
-                              {'v': 'Q847102',
-                               'ref': {'9d96507726508344ef1b8f59092fb350171b3d99':
-                                           {('P248', 'Q29458763'), ('P594', 'ENSG00000123374')}},
-                               'qual': {('P659', 'Q21067546'), ('P659', 'Q20966585')},
-                               }
-                          }
-                     }
-
 
 class FastRunContainer(object):
     def __init__(self, base_data_type, engine, mediawiki_api_url=None, sparql_endpoint_url=None, wikibase_url=None,
-                 concept_base_uri=None, base_filter=None, use_refs=False, ref_handler=None, case_insensitive=False,
-                 debug=False):
+                 base_filter=None, use_refs=False, ref_handler=None, case_insensitive=False, debug=False):
         self.prop_data = {}
         self.loaded_langs = {}
         self.statements = []
@@ -34,7 +24,6 @@ class FastRunContainer(object):
         self.mediawiki_api_url = config['MEDIAWIKI_API_URL'] if mediawiki_api_url is None else mediawiki_api_url
         self.sparql_endpoint_url = config['SPARQL_ENDPOINT_URL'] if sparql_endpoint_url is None else sparql_endpoint_url
         self.wikibase_url = config['WIKIBASE_URL'] if wikibase_url is None else wikibase_url
-        self.concept_base_uri = config['CONCEPT_BASE_URI'] if concept_base_uri is None else concept_base_uri
         self.case_insensitive = case_insensitive
         self.debug = debug
         self.reconstructed_statements = []
@@ -43,18 +32,29 @@ class FastRunContainer(object):
 
         if base_filter and any(base_filter):
             self.base_filter = base_filter
-
             for k, v in self.base_filter.items():
+                ks = []
+                if k.count('/') == 1:
+                    ks = k.split('/')
                 if v:
-                    self.base_filter_string += '?item wdt:{0} wd:{1} . \n'.format(k, v)
+                    if ks:
+                        self.base_filter_string += '?item <{wb_url}/prop/direct/{prop_nr1}>/<{wb_url}/prop/direct/{prop_nr2}>* <{wb_url}/entity/{entity}> .\n'.format(wb_url=self.wikibase_url, prop_nr1=ks[0], prop_nr2=ks[1], entity=v)
+                    else:
+                        self.base_filter_string += '?item <{wb_url}/prop/direct/{prop_nr}> <{wb_url}/entity/{entity}> .\n'.format(wb_url=self.wikibase_url, prop_nr=k, entity=v)
+
                 else:
-                    self.base_filter_string += '?item wdt:{0} ?zz . \n'.format(k)
+                    if ks:
+                        self.base_filter_string += '?item <{wb_url}/prop/direct/{prop_nr1}>/<{wb_url}/prop/direct/{prop_nr2}>* ?zz{prop_nr1}{prop_nr2} .\n'.format(wb_url=self.wikibase_url, prop_nr1=ks[0], prop_nr2=ks[1], entity=v)
+                    else:
+                        self.base_filter_string += '?item <{wb_url}/prop/direct/{prop_nr}> ?zz{prop_nr} .\n'.format(wb_url=self.wikibase_url, prop_nr=k, entity=v)
 
     def reconstruct_statements(self, qid):
         reconstructed_statements = []
+
         if qid not in self.prop_data:
             self.reconstructed_statements = reconstructed_statements
             return reconstructed_statements
+
         for prop_nr, dt in self.prop_data[qid].items():
             # get datatypes for qualifier props
             q_props = set(chain(*[[x[0] for x in d['qual']] for d in dt.values()]))
@@ -69,7 +69,10 @@ class FastRunContainer(object):
                 for q in d['qual']:
                     f = [x for x in self.base_data_type.__subclasses__() if x.DTYPE ==
                          self.prop_dt_map[q[0]]][0]
-                    qualifiers.append(f(q[1], prop_nr=q[0], is_qualifier=True))
+                    if self.prop_dt_map[q[0]] == 'quantity' and q[2] != '1':
+                        qualifiers.append(f(q[1], prop_nr=q[0], is_qualifier=True, unit=q[2]))
+                    else:
+                        qualifiers.append(f(q[1], prop_nr=q[0], is_qualifier=True))
 
                 references = []
                 for ref_id, refs in d['ref'].items():
@@ -84,8 +87,7 @@ class FastRunContainer(object):
                      self.prop_dt_map[prop_nr]][0]
                 if self.prop_dt_map[prop_nr] == 'quantity' and d['unit'] != '1':
                     reconstructed_statements.append(
-                        f(d['v'], prop_nr=prop_nr, qualifiers=qualifiers, references=references, unit=d['unit'],
-                          concept_base_uri=self.concept_base_uri))
+                        f(d['v'], prop_nr=prop_nr, qualifiers=qualifiers, references=references, unit=d['unit']))
                 else:
                     reconstructed_statements.append(
                         f(d['v'], prop_nr=prop_nr, qualifiers=qualifiers, references=references))
@@ -147,7 +149,7 @@ class FastRunContainer(object):
         # if not, a write is required no matter what
         if not len(matching_qids) == 1:
             if self.debug:
-                print('no matches')
+                print('no matches ({})'.format(len(matching_qids)))
             return True
 
         qid = matching_qids.pop()
@@ -215,7 +217,8 @@ class FastRunContainer(object):
             bool_vec = []
             for x in tmp_rs:
                 if (x.get_value() == date.get_value() or (
-                        self.case_insensitive and x.get_value().casefold() == date.get_value().casefold())) and x.get_prop_nr() not in del_props:
+                        self.case_insensitive and x.get_value().casefold() == date.get_value().casefold())) and \
+                        x.get_prop_nr() not in del_props:
                     if self.use_refs and self.ref_handler:
                         to_be = copy.deepcopy(x)
                         self.ref_handler(to_be, date)
@@ -239,10 +242,9 @@ class FastRunContainer(object):
                     if date == x and x.get_prop_nr() not in del_props:
                         print(x.get_prop_nr(), x.get_value(), [z.get_value() for z in x.get_qualifiers()])
                         print(date.get_prop_nr(), date.get_value(), [z.get_value() for z in date.get_qualifiers()])
-                    else:
-                        if x.get_prop_nr() == date.get_prop_nr():
-                            print(x.get_prop_nr(), x.get_value(), [z.get_value() for z in x.get_qualifiers()])
-                            print(date.get_prop_nr(), date.get_value(), [z.get_value() for z in date.get_qualifiers()])
+                    elif x.get_prop_nr() == date.get_prop_nr():
+                        print(x.get_prop_nr(), x.get_value(), [z.get_value() for z in x.get_qualifiers()])
+                        print(date.get_prop_nr(), date.get_value(), [z.get_value() for z in date.get_qualifiers()])
 
             if not any(bool_vec):
                 if self.debug:
@@ -250,6 +252,8 @@ class FastRunContainer(object):
                     print('fast run failed at', date.get_prop_nr())
                 write_required = True
             else:
+                if self.debug:
+                    print('fast run success')
                 tmp_rs.pop(bool_vec.index(True))
 
         if len(tmp_rs) > 0:
@@ -279,7 +283,7 @@ class FastRunContainer(object):
     def get_language_data(self, qid, lang, lang_data_type):
         """
         get language data for specified qid
-        :param qid:
+        :param qid:  Wikibase item id
         :param lang: language code
         :param lang_data_type: 'label', 'description' or 'aliases'
         :return: list of strings
@@ -296,22 +300,29 @@ class FastRunContainer(object):
             all_lang_strings = ['']
         return all_lang_strings
 
-    def check_language_data(self, qid, lang_data, lang, lang_data_type):
+    def check_language_data(self, qid, lang_data, lang, lang_data_type, if_exists='APPEND'):
         """
         Method to check if certain language data exists as a label, description or aliases
+        :param qid: Wikibase item id
         :param lang_data: list of string values to check
         :type lang_data: list
         :param lang: language code
         :type lang: str
         :param lang_data_type: What kind of data is it? 'label', 'description' or 'aliases'?
-        :return:
+        :param if_exists: If aliases already exist, APPEND or REPLACE
+        :return: boolean
         """
-        all_lang_strings = set(x.strip().lower() for x in self.get_language_data(qid, lang, lang_data_type))
+        all_lang_strings = set(x.strip().casefold() for x in self.get_language_data(qid, lang, lang_data_type))
 
-        for s in lang_data:
-            if s.strip().lower() not in all_lang_strings:
-                print('fastrun failed at label: {}, string: {}'.format(lang_data_type, s))
-                return True
+        if if_exists == 'REPLACE':
+            return not collections.Counter(all_lang_strings) == collections.Counter(map(lambda x: x.casefold(),
+                                                                                        lang_data))
+        else:
+            for s in lang_data:
+                if s.strip().casefold() not in all_lang_strings:
+                    if self.debug:
+                        print('fastrun failed at: {}, string: {}'.format(lang_data_type, s))
+                    return True
 
         return False
 
@@ -324,19 +335,20 @@ class FastRunContainer(object):
         `prop_nr` is needed to get the property datatype to determine how to format the value
 
         `r` is a list of dicts. The keys are:
+            sid: statement ID
             item: the subject. the item this statement is on
             v: the object. The value for this statement
-            sid: statement ID
+            unit: property unit
             pq: qualifier property
             qval: qualifier value
+            qunit: qualifier unit
             ref: reference ID
             pr: reference property
             rval: reference value
-            unit: property unit
         """
         prop_dt = self.get_prop_datatype(prop_nr)
         for i in r:
-            for value in {'item', 'sid', 'pq', 'pr', 'ref', 'unit'}:
+            for value in {'item', 'sid', 'pq', 'pr', 'ref', 'unit', 'qunit'}:
                 if value in i:
                     # these are always URIs for the local wikibase
                     i[value] = i[value]['value'].split('/')[-1]
@@ -373,6 +385,8 @@ class FastRunContainer(object):
                 qual_prop_dt = self.get_prop_datatype(prop_nr=i['pq'])
                 if i['qval']['type'] == 'uri' and qual_prop_dt == 'wikibase-item':
                     i['qval'] = i['qval']['value'].split('/')[-1]
+                elif i['qval']['type'] == 'literal' and qual_prop_dt == 'quantity':
+                    i['qval'] = self.format_amount(i['qval']['value'])
                 else:
                     i['qval'] = i['qval']['value']
 
@@ -384,7 +398,8 @@ class FastRunContainer(object):
                 else:
                     i['rval'] = i['rval']['value']
 
-    def format_amount(self, amount):
+    @staticmethod
+    def format_amount(amount):
         # Remove .0 by casting to int
         if float(amount) % 1 == 0:
             amount = int(float(amount))
@@ -414,7 +429,10 @@ class FastRunContainer(object):
             if 'qual' not in self.prop_data[qid][prop_nr][i['sid']]:
                 self.prop_data[qid][prop_nr][i['sid']]['qual'] = set()
             if 'pq' in i and 'qval' in i:
-                self.prop_data[qid][prop_nr][i['sid']]['qual'].add((i['pq'], i['qval']))
+                if 'qunit' in i:
+                    self.prop_data[qid][prop_nr][i['sid']]['qual'].add((i['pq'], i['qval'], i['qunit']))
+                else:
+                    self.prop_data[qid][prop_nr][i['sid']]['qual'].add((i['pq'], i['qval'], '1'))
 
             if 'ref' not in self.prop_data[qid][prop_nr][i['sid']]:
                 self.prop_data[qid][prop_nr][i['sid']]['ref'] = dict()
@@ -428,21 +446,17 @@ class FastRunContainer(object):
             if 'unit' in i:
                 self.prop_data[qid][prop_nr][i['sid']]['unit'] = i['unit']
 
-    def _query_data_refs(self, prop_nr):
+    def _query_data(self, prop_nr):
         page_size = 10000
         page_count = 0
         num_pages = None
         if self.debug:
             # get the number of pages/queries so we can show a progress bar
-            query = """PREFIX wd: <{0}/entity/>
-            PREFIX wdt: <{0}/prop/direct/>
-            PREFIX p: <{0}/prop/>
-            PREFIX ps: <{0}/prop/statement/>
-
+            query = """
             SELECT (COUNT(?item) as ?c) where {{
-                  {1}
-                  ?item p:{2} ?sid .
-            }}""".format(self.wikibase_url, self.base_filter_string, prop_nr)
+                  {base_filter}
+                  ?item <{wb_url}/prop/{prop_nr}> ?sid .
+            }}""".format(wb_url=self.wikibase_url, base_filter=self.base_filter_string, prop_nr=prop_nr)
 
             if self.debug:
                 print(query)
@@ -452,84 +466,106 @@ class FastRunContainer(object):
             num_pages = (int(count) // page_size) + 1
             print("Query {}: {}/{}".format(prop_nr, page_count, num_pages))
         while True:
-            query = """
-                PREFIX wd: <**wikibase_url**/entity/>
-                PREFIX wdt: <**wikibase_url**/prop/direct/>
-                PREFIX p: <**wikibase_url**/prop/>
-                PREFIX ps: <**wikibase_url**/prop/statement/>
-                #Tool: wbi_core fastrun
-                SELECT ?item ?qval ?pq ?sid ?v ?ref ?pr ?rval WHERE {
-                  {
-                    SELECT ?item ?v ?sid where {
-                      **base_filter_string**
-                      ?item p:**prop_nr** ?sid .
-                      ?sid ps:**prop_nr** ?v .
-                    } GROUP BY ?item ?v ?sid
-                    ORDER BY ?sid
-                    OFFSET **offset**
-                    LIMIT **page_size**
-                  }
-                  OPTIONAL {
-                    ?sid ?pq ?qval .
-                    [] wikibase:qualifier ?pq
-                  }
-                  OPTIONAL {
-                    ?sid prov:wasDerivedFrom ?ref .
-                    ?ref ?pr ?rval .
-                    [] wikibase:reference ?pr
-                  }
-                }""".replace("**offset**", str(page_count * page_size)). \
-                replace("**base_filter_string**", self.base_filter_string). \
-                replace("**prop_nr**", prop_nr).replace("**page_size**", str(page_size)). \
-                replace("**wikibase_url**", self.wikibase_url)
+            if self.use_refs:
+                query = '''
+                    #Tool: wbi_fastrun _query_data_refs
+                    SELECT ?sid ?item ?v ?unit ?pq ?qval ?qunit ?ref ?pr ?rval
+                    WHERE
+                    {{
+                      {base_filter}
+
+                      # Get amount and unit for the statement
+                      ?item <{wb_url}/prop/{prop_nr}> ?sid .
+                      {{
+                        <{wb_url}/entity/{prop_nr}> wikibase:propertyType ?property_type .
+                        FILTER (?property_type != wikibase:Quantity)
+                        ?sid <{wb_url}/prop/statement/{prop_nr}> ?v .
+                      }}
+                      UNION
+                      {{
+                        ?sid <{wb_url}/prop/statement/value/{prop_nr}> [wikibase:quantityAmount ?v; wikibase:quantityUnit ?unit] .
+                      }}
+
+                      # Get qualifiers
+                      OPTIONAL
+                      {{
+                        {{
+                          # Get simple values for qualifiers which are not of type quantity
+                          ?sid ?propQualifier ?qval .
+                          ?pq wikibase:qualifier ?propQualifier .
+                          ?pq wikibase:propertyType ?qualifer_property_type .
+                          FILTER (?qualifer_property_type != wikibase:Quantity)
+                        }}
+                        UNION
+                        {{
+                          # Get amount and unit for qualifiers of type quantity
+                          ?sid ?pqv [wikibase:quantityAmount ?qval; wikibase:quantityUnit ?qunit] .
+                          ?pq wikibase:qualifierValue ?pqv .
+                        }}
+                      }}
+
+                      # get references
+                      OPTIONAL {{
+                        ?sid prov:wasDerivedFrom ?ref .
+                        ?ref ?pr ?rval .
+                        [] wikibase:reference ?pr
+                      }}
+                    }} ORDER BY ?sid OFFSET {offset} LIMIT {page_size}
+                    '''.format(wb_url=self.wikibase_url, base_filter=self.base_filter_string, prop_nr=prop_nr,
+                               offset=str(page_count * page_size), page_size=str(page_size))
+            else:
+                query = '''
+                    #Tool: wbi_fastrun _query_data
+                    SELECT ?sid ?item ?v ?unit ?pq ?qval ?qunit
+                    WHERE
+                    {{
+                      {base_filter}
+
+                      # Get amount and unit for the statement
+                      ?item <{wb_url}/prop/{prop_nr}> ?sid .
+                      {{
+                        <{wb_url}/entity/{prop_nr}> wikibase:propertyType ?property_type .
+                        FILTER (?property_type != wikibase:Quantity)
+                        ?sid <{wb_url}/prop/statement/{prop_nr}> ?v .
+                      }}
+                      UNION
+                      {{
+                        ?sid <{wb_url}/prop/statement/value/{prop_nr}> [wikibase:quantityAmount ?v; wikibase:quantityUnit ?unit] .
+                      }}
+
+                      # Get qualifiers
+                      OPTIONAL
+                      {{
+                        {{
+                          # Get simple values for qualifiers which are not of type quantity
+                          ?sid ?propQualifier ?qval .
+                          ?pq wikibase:qualifier ?propQualifier .
+                          ?pq wikibase:propertyType ?qualifer_property_type .
+                          FILTER (?qualifer_property_type != wikibase:Quantity)
+                        }}
+                        UNION
+                        {{
+                          # Get amount and unit for qualifiers of type quantity
+                          ?sid ?pqv [wikibase:quantityAmount ?qval; wikibase:quantityUnit ?qunit] .
+                          ?pq wikibase:qualifierValue ?pqv .
+                        }}
+                      }}
+                    }} ORDER BY ?sid OFFSET {offset} LIMIT {page_size}
+                    '''.format(wb_url=self.wikibase_url, base_filter=self.base_filter_string, prop_nr=prop_nr,
+                               offset=str(page_count * page_size), page_size=str(page_size))
 
             if self.debug:
                 print(query)
 
-            results = self.engine.execute_sparql_query(query, endpoint=self.sparql_endpoint_url)['results']['bindings']
+            results = self.engine.execute_sparql_query(query=query, endpoint=self.sparql_endpoint_url)['results']['bindings']
             self.format_query_results(results, prop_nr)
             self.update_frc_from_query(results, prop_nr)
             page_count += 1
             if num_pages:
                 print("Query {}: {}/{}".format(prop_nr, page_count, num_pages))
-            if len(results) == 0:
+            if len(results) == 0 or len(results) < page_size:
                 break
 
-    def _query_data(self, prop_nr):
-        if self.use_refs:
-            self._query_data_refs(prop_nr)
-        else:
-            query = '''
-                PREFIX wd: <{0}/entity/>
-                PREFIX wdt: <{0}/prop/direct/>
-                PREFIX p: <{0}/prop/>
-                PREFIX ps: <{0}/prop/statement/>
-                PREFIX psv: <{0}/prop/statement/value/>
-                #Tool: wbi_core fastrun
-                select ?item ?qval ?pq ?sid ?v ?unit where {{
-                  {1}
-
-                  ?item p:{2} ?sid .
-
-                  ?sid ps:{2} ?v .
-                  OPTIONAL {{
-                    ?sid ?pq ?qval .
-                    [] wikibase:qualifier ?pq
-                  }}
-                  OPTIONAL {{
-                    ?sid psv:{2} ?valuenode .
-                    ?valuenode wikibase:quantityUnit ?unit
-                  }}
-                }}
-                '''.format(self.wikibase_url, self.base_filter_string, prop_nr)
-
-            if self.debug:
-                print(query)
-
-            r = self.engine.execute_sparql_query(query=query, endpoint=self.sparql_endpoint_url)['results']['bindings']
-            self.format_query_results(r, prop_nr)
-            self.update_frc_from_query(r, prop_nr)
-
     def _query_lang(self, lang, lang_data_type):
         """
 
@@ -545,19 +581,15 @@ class FastRunContainer(object):
         }
 
         query = '''
-        PREFIX wd: <{0}/entity/>
-        PREFIX wdt: <{0}/prop/direct/>
-        PREFIX p: <{0}/prop/>
-        PREFIX ps: <{0}/prop/statement/>
-        #Tool: wbi_core fastrun
+        #Tool: wbi_fastrun _query_lang
         SELECT ?item ?label WHERE {{
-            {1}
+            {base_filter}
 
             OPTIONAL {{
-                ?item {2} ?label FILTER (lang(?label) = "{3}") .
+                ?item {lang_data_type} ?label FILTER (lang(?label) = "{lang}") .
             }}
         }}
-        '''.format(self.wikibase_url, self.base_filter_string, lang_data_type_dict[lang_data_type], lang)
+        '''.format(base_filter=self.base_filter_string, lang_data_type=lang_data_type_dict[lang_data_type], lang=lang)
 
         if self.debug:
             print(query)
@@ -589,9 +621,8 @@ class FastRunContainer(object):
         self.rev_lookup = defaultdict(set)
         self.rev_lookup_ci = defaultdict(set)
 
-    """A mixin implementing a simple __repr__."""
-
     def __repr__(self):
+        """A mixin implementing a simple __repr__."""
         return "<{klass} @{id:x} {attrs}>".format(
             klass=self.__class__.__name__,
             id=id(self) & 0xFFFFFF,
